"""example add-on: sort direct children of node
"""
import logging
from PySide6.QtWidgets import QMessageBox
from pasta_eln.backendWorker.worker import Task
from pasta_eln.miscTools import getHierarchy

# The following two variables are mandatory
description  = 'Sort children'  #short description that is shown in the menu
reqParameter = {} #possibility for required parameters: like API-key, etc. {'API': 'text'}


def main(comm, hierStack, widget, parameter={}):
  """ main function: has to exist and is called by the menu
  Args:
    comm (Communicate): communicate-backend
    hierStack (str): node in hierarchy to start the creation
    widget (QWidget): allows to create new gui dialogs
    parameter (dict): ability to pass parameters

  Returns:
      bool: success
  """
  maxNumberSorting = 100
  res = QMessageBox.question(widget, 'Sort direction', 'Sort direct children in ascending order? '
          'If you click no, sort in descending order.',
          buttons=QMessageBox.StandardButtons(QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No | QMessageBox.StandardButton.Cancel),
          defaultButton=QMessageBox.StandardButton.Yes)
  if res == QMessageBox.StandardButton.Cancel:
    return False
  ascendingFlag = res == QMessageBox.StandardButton.Yes
  hierStackI = hierStack.split('/')

  # Issue: previous sorting steps influence later steps. Hence after one sorting step, always get the updated order and look for changes
  # One could use a 'while True' loop but that would risk erroneous endless loops. Better, max. number and then user has to sort again.
  for idx in range(maxNumberSorting):
    hierarchy, _ = getHierarchy(comm, hierStack)
    noChange = True
    for revIdx, node in enumerate(sorted(hierarchy.children, key=lambda n: n.name.lower(), reverse=ascendingFlag)):
      idx = len(hierarchy.children) - 1 - revIdx
      if idx == node.childNum:
        continue
      comm.uiRequestTask.emit(Task.MOVE_LEAVES, {'docID':node.id, 'stackOld':hierStackI,'stackNew':hierStackI,
                                                 'childOld':node.childNum, 'childNew':idx})
      noChange = False
      break
    if noChange:
      break
    if idx == maxNumberSorting-1:
      logging.info('Only sorted %s times according to max. number of sorting', maxNumberSorting)


  return True



""" ReorderColumnDelegate for the table views """
#  PASTA-ELN and all its sub-parts are covered by the MIT license
#
#  Copyright (c) 2023
#
#  Author: Jithu Murugan
#  Filename: reorder_column_delegate.py
#
#  You should have received a copy of the license with this file. Please refer the license file for more information
from typing import Any, Union
import qtawesome as qta
from PySide6.QtCore import QAbstractItemModel, QEvent, QModelIndex, QPersistentModelIndex, QSize
from PySide6.QtGui import QPainter
from PySide6.QtWidgets import (QApplication, QPushButton, QStyle, QStyledItemDelegate, QStyleOptionButton,
                               QStyleOptionViewItem, QWidget)
from ..data_hierarchy.utility_functions import is_click_within_bounds
from .terminology_lookup_dialog import TerminologyLookupDialog


class LookupDelegate(QStyledItemDelegate):
  """
  Delegate for creating the icons for the re-order column in the data hierarchy editor tables
  """
  def __init__(self, parent:Any = None):
    super().__init__(parent)
    self.button = QPushButton()


  def paint(self,
            painter: QPainter,
            option: QStyleOptionViewItem,
            index: Union[QModelIndex, QPersistentModelIndex]) -> None:
    """
    Draws the re-order button within the cell represented by index
    Args:
      painter (QPainter): Painter instance for painting the button
      option (QStyleOptionViewItem): Style option for the cell represented by index
      index (Union[QModelIndex, QPersistentModelIndex]): Table cell index
    """
    opt = QStyleOptionButton()
    opt.state = QStyle.StateFlag.State_Active | QStyle.StateFlag.State_Enabled    # type: ignore[attr-defined]
    opt.rect = option.rect                                                        # type: ignore[attr-defined]
    opt.icon = qta.icon('fa5s.search', scale_factor=1.0)                          # type: ignore[attr-defined]
    opt.iconSize = QSize(15, 15)                                                  # type: ignore[attr-defined]
    QApplication.style().drawControl(QStyle.ControlElement.CE_PushButton, opt, painter, self.button)
    return


  def createEditor(self,
                   parent: QWidget,
                   option: QStyleOptionViewItem,
                   index: Union[QModelIndex, QPersistentModelIndex]) -> QWidget:
    """
    Disable the editor for the whole re-order column by simply returning None
    Args:
      parent (QWidget): Parent table view
      option (QStyleOptionViewItem): Style option for the cell represented by index
      index (Union[QModelIndex, QPersistentModelIndex]): Cell index
    """
    return None                                                                   # type: ignore[return-value]


  def editorEvent(self,
                  event: QEvent,
                  model: QAbstractItemModel,
                  option: QStyleOptionViewItem,
                  index: Union[QModelIndex, QPersistentModelIndex]) -> bool:
    """
    In case of mouse click event, the re_order_signal is emitted for the respective table cell position
    Args:
      event (QEvent): The editor event information
      model (QAbstractItemModel): Model data representing the table view
      option (QStyleOptionViewItem): QStyleOption for the table cell
      index (Union[QModelIndex, QPersistentModelIndex]): Table cell index

    Returns (bool): True/False
    """
    # callback function to set the PURL
    def setPURL(iris: list[str]) -> None:
      """ Callback function to set the PURL
      Args:
        iris (list[str]): List of IRIs
      """
      indexPurl = index.model().index(index.row(), 2)
      index.model().setData(indexPurl, ', '.join(iris))
      return

    keyText = index.model().index(index.row(), 0).data()
    labelText = index.model().index(index.row(), 1).data()
    if is_click_within_bounds(event, option) and (keyText or labelText):
      ui = TerminologyLookupDialog(labelText or keyText, setPURL)
      ui.instance.show()
      return True
    return False
